<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Games;
use App\Models\GameRates;
use App\Models\StarlineGames;
use App\Events\RefreshGame;
use App\Events\RefreshStarlineGame;
use App\Events\GameTimeChange;
use App\Models\StarlineGameRates;
use Yajra\DataTables\DataTables;

class GamesController extends Controller
{
    public function gameName(Request $request)
    {

        return view('games.gameName');
    }

    public function addGame(Request $request)
    {

        $days = array('Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday');
        $market_time = [];
        foreach ($days as $value) {

            $market_time[$value] = array('market_open' => '1', "open_time" => $request->open_time, "close_time" => $request->close_time);
        }

        $game = Games::create([

            'game_name' => $request->game_name,
            'game_name_hindi' => $request->game_name_hindi,
            'market_time' => json_encode($market_time),

        ]);
        $game->save();
        return response()->json(['error' => false, 'status' => 'success']);
    }

    public function gamesList(Request $request)
    {

        $games = Games::all();//where('role','=','client')->get();
        return Datatables::of($games)
            ->addIndexColumn()
            ->editColumn('status', function ($row) {
                if ($row->status == 'active') {
                    $status = '<a role="button" class="activeDeactive" id="success-' . $row['id'] . '-games-id-status"><span class="badge badge-pill badge-soft-success font-size-12">Yes</span></a>';
                } else {
                    $status = '<a role="button" class="activeDeactive" id="danger-' . $row['id'] . '-games-id-status"><span class="badge badge-pill badge-soft-danger font-size-12">No</span></a>';
                }
                return $status;
            })
            ->editColumn('created_at', function ($row) {

                return date('Y-m-d', strtotime($row['created_at']));
            })
            ->addColumn('open_time', function ($row) {

                $market_time = json_decode($row->market_time, true);

                return date('h:i A', strtotime($market_time[date('l')]['open_time']));
            })
            ->addColumn('close_time', function ($row) {

                $market_time = json_decode($row->market_time, true);

                return date('h:i A', strtotime($market_time[date('l')]['close_time']));
            })
            ->addColumn('market_status', function ($row) {

                $market_time = json_decode($row->market_time, true);
                if ($market_time[date('l')]['market_open'] == '0') {
                    $badge = '<span class="badge badge-pill badge-soft-danger font-size-12">Close Today</span>';
                } else {
                    $badge = '<span class="badge badge-pill badge-soft-success font-size-12">Open Today</span>';
                }
                return $badge;
            })
            ->rawColumns(['status', 'close_time', 'open_time', 'market_status'])
            ->make(true);
    }

    public function editGame($id)
    {

        $game = Games::find($id);
        $view = \View::make('games.editGame', compact('game'));
        $html = $view->render();
        return $html;
    }

    public function updateGame(Request $request)
    {

        $game = Games::find($request->game_id);
        $game->game_name = $request->game_name;
        $game->game_name_hindi = $request->game_name_hindi;
        $game->save();
        return response()->json(['msg' => 'updated successfully!', 'status' => 'update']);
    }

    public function offDay($id)
    {

        $game = Games::find($id);
        $market_time = json_decode($game->market_time, true);
        $game_id = $game->id;
        // dd($market_time);
        $view = \View::make('games.market_time', compact('market_time', 'game_id'));
        $html = $view->render();
        return $html;
    }

    public function addOffDay(Request $request)
    {
        //dd($request);
        $game = Games::find($request->game_id);
        $days = array('Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday');
        $market_time = [];
        foreach ($days as $key => $value) {

            if (in_array($value, $request->day)) {
                $open = '1';
            } else {
                $open = '0';
            }

            $market_time[$value] = array('market_open' => $open, "open_time" => $request['open_time'][$key], "close_time" => $request['close_time'][$key]);
        }

        $game->market_time = json_encode($market_time);
        $game->save();
        broadcast(new GameTimeChange());
        return response()->json(['msg' => 'updated successfully!', 'status' => 'success']);
    }

    public function gameRates(Request $request)
    {
        $rates = GameRates::find(1);
        return view('games.gameRates', compact('rates'));
    }

    public function addGameRates(Request $request)
    {

        $rates = GameRates::find(1);
        $rates->single_digit_value_one = $request->single_digit_1;
        $rates->single_digit_value_two = $request->single_digit_2;
        $rates->jodi_digit_value_one = $request->jodi_digit_1;
        $rates->jodi_digit_value_two = $request->jodi_digit_2;
        $rates->single_pana_value_one = $request->single_pana_1;
        $rates->single_pana_value_two = $request->single_pana_2;
        $rates->double_pana_value_one = $request->double_pana_1;
        $rates->double_pana_value_two = $request->double_pana_2;
        $rates->triple_pana_value_one = $request->tripple_pana_1;
        $rates->triple_pana_value_two = $request->tripple_pana_2;
        $rates->half_sangam_value_one = $request->half_sangam_1;
        $rates->half_sangam_value_two = $request->half_sangam_2;
        $rates->full_sangam_value_one = $request->full_sangam_1;
        $rates->full_sangam_value_two = $request->full_sangam_2;
        $rates->save();
        return response()->json(['msg' => '<p class="alert alert-success">Game Rates Successfully Updated.</p>', 'status' => 'success']);
    }

    public function starlineGameName(Request $request)
    {

        return view('starline.starlineGameName');
    }

    public function starlineAddGame(Request $request)
    {

        $game = StarlineGames::create([

            'game_name' => $request->game_name,
            'game_name_hindi' => $request->game_name_hindi,
            'open_time' => $request->open_time,
            'market_status' => 'active'

        ]);
        $game->save();
        return response()->json(['error' => false, 'status' => 'success']);
    }

    public function starlineGamesList(Request $request)
    {

        $games = StarlineGames::orderBy('open_time', 'desc')->get();
        return Datatables::of($games)
            ->addIndexColumn()
            ->editColumn('status', function ($row) {
                if ($row->status == 'active') {
                    $status = '<a role="button" class="activeDeactive" id="success-' . $row['id'] . '-starline_games-id-status"><span class="badge badge-pill badge-soft-success font-size-12">Yes</span></a>';
                } else {
                    $status = '<a role="button" class="activeDeactive" id="danger-' . $row['id'] . '-starline_games-id-status"><span class="badge badge-pill badge-soft-danger font-size-12">No</span></a>';
                }
                return $status;
            })
            ->editColumn('created_at', function ($row) {

                return date('Y-m-d', strtotime($row['created_at']));
            })
            ->addColumn('market_status', function ($row) {

                //$market_time=json_decode($row->market_time,true);
                if ($row['market_open'] == 'blocked') {
                    $badge = '<div id="market_open' . $row['id'] . '"><span class="badge badge-pill badge-soft-danger font-size-12">Market Closed</span></div>';
                } else {
                    $badge = '<div id="market_open' . $row['id'] . '"><span class="badge badge-pill badge-soft-success font-size-12">Market Open</span></div>';
                }
                return $badge;
            })
            ->rawColumns(['status', 'market_status'])
            ->make(true);
    }

    public function starlineEditGame($id)
    {

        $game = StarlineGames::find($id);
        $view = \View::make('starline.editGame', compact('game'));
        $html = $view->render();
        return $html;
    }

    public function starlineUpdateGame(Request $request)
    {
        //dd($request);
        $game = StarlineGames::find($request->game_id);
        //dd($game);
        $game->game_name = $request->game_name;
        $game->game_name_hindi = $request->game_name_hindi;
        $game->open_time = $request->open_time;
        $game->save();
        //dd($game);
        return response()->json(['msg' => 'updated successfully!', 'status' => 'update']);
    }

    public function starlineGameRates(Request $request)
    {

        $rates = StarlineGameRates::find(1);
        //dd($rates);
        return view('starline.starlineGameRate', compact('rates'));
    }

    public function starlineAddGameRates(Request $request)
    {

        $rates = StarlineGameRates::find(1);
        $rates->single_digit_value_one = $request->single_digit_1;
        $rates->single_digit_value_two = $request->single_digit_2;
        $rates->single_pana_value_one = $request->single_pana_1;
        $rates->single_pana_value_two = $request->single_pana_2;
        $rates->double_pana_value_one = $request->double_pana_1;
        $rates->double_pana_value_two = $request->double_pana_2;
        $rates->triple_pana_value_one = $request->tripple_pana_1;
        $rates->triple_pana_value_two = $request->tripple_pana_2;
        $rates->save();
        return response()->json(['msg' => '<p class="alert alert-success">Game Rates Successfully Updated.</p>', 'status' => 'success']);
    }

}
