<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use App\Models\User;
use App\Models\TransactionHistory;
use App\Models\Notice;
use App\Models\UserBid;
use App\Models\Withdraw;
use App\Models\ImageSlide;
use App\Models\UserInquiry;
use App\Events\UserBattingOff;
use App\Events\UserBlock;
use App\Events\RefreshGame;
use App\Events\RefreshStarlineGame;
use Yajra\DataTables\DataTables;
use DB;
use Hash;
use App\Models\HowtoPlay;

class UserController extends Controller
{
    public function users(Request $request)
    {

        return view('users.users');
    }

    public function usersList(Request $request)
    {

        $limit = $request->length;
        $start = $request->start;
        $keyword = $request->search['value'];
        $count = User::where('role', '=', 'client')->count();
        if ($keyword == '') {
            $users = User::where('role', '=', 'client')->orderBy('id', 'desc')->skip($start)->take($limit)->get();
            //dd($users);


        } else {
            $users = User::where('role', '=', 'client')->orderBy('id', 'desc')
                ->where(function ($query) use ($keyword) {
                    $query->where('username', 'like', '%' . $keyword . '%')
                        ->orWhere('username', 'like', '%' . $keyword . '%')
                        ->orWhere('phone_number', 'like', '%' . $keyword . '%')
                        ->orWhere('email', 'like', '%' . $keyword . '%')
                        ->orWhere('available_balance', 'like', '%' . $keyword . '%');
                })->skip($start)->take($limit)->get();

            $filter_count = User::where('role', '=', 'client')->orderBy('id', 'desc')
                ->where(function ($query) use ($keyword) {
                    $query->where('username', 'like', '%' . $keyword . '%')
                        ->orWhere('username', 'like', '%' . $keyword . '%')
                        ->orWhere('phone_number', 'like', '%' . $keyword . '%')
                        ->orWhere('email', 'like', '%' . $keyword . '%')
                        ->orWhere('available_balance', 'like', '%' . $keyword . '%');
                })->count();
        }

        return Datatables::of($users)
            ->addIndexColumn()
            ->setOffset($start)
            ->editColumn('username', function ($row) {
                $link = '<a href="' . url('/') . '/admin/' . $row['id'] . '/userDetails" style="color:blue;">' . $row['username'] . '</a>';
                return $link;
            })
            ->editColumn('created_at', function ($row) {

                return date('Y-m-d', strtotime($row['created_at']));
            })
            ->editColumn('status', function ($row) {
                if ($row->status == 'active') {
                    $status = '<a role="button" class="activeDeactive" id="success-' . $row['id'] . '-users-id-status"><span class="badge badge-pill badge-soft-success font-size-12">Yes</span></a>';
                } else {
                    $status = '<a role="button" class="activeDeactive" id="danger-' . $row['id'] . '-users-id-status"><span class="badge badge-pill badge-soft-danger font-size-12">No</span></a>';
                }
                return $status;
            })
            ->editColumn('betting', function ($row) {
                if ($row->betting == 'yes') {
                    $status = '<a role="button" class="activeDeactive" id="success-' . $row['id'] . '-users-id-betting"><span class="badge badge-pill badge-soft-success font-size-12">Yes</span></a>';
                } else {
                    $status = '<a role="button" class="activeDeactive" id="danger-' . $row['id'] . '-users-id-betting"><span class="badge badge-pill badge-soft-danger font-size-12">No</span></a>';
                }
                return $status;
            })
            ->editColumn('transfer', function ($row) {
                if ($row->transfer == 'yes') {
                    $status = '<a role="button" class="activeDeactive" id="success-' . $row['id'] . '-users-id-transfer"><span class="badge badge-pill badge-soft-success font-size-12">Yes</span></a>';
                } else {
                    $status = '<a role="button" class="activeDeactive" id="danger-' . $row['id'] . '-users-id-transfer"><span class="badge badge-pill badge-soft-danger font-size-12">No</span></a>';
                }
                return $status;
            })
            ->addColumn('view_details', function ($row) {
                $btn = '<a href="' . url('/') . '/admin/' . $row['id'] . '/userDetails" class="mr-3 text-primary" data-toggle="tooltip" data-placement="top" title="" data-original-title="View Details" aria-describedby="tooltip224897"><i class="mdi mdi-eye font-size-18"></i></a>';


                return $btn;
            })
            ->rawColumns(['#', 'view_details', 'username', 'status', 'betting', 'transfer'])
            ->with([
                "recordsTotal" => $count,
                "recordsFiltered" => (isset($filter_count)) ? $filter_count : $count,
            ])
            ->make(true);


    }

    public function unapprovedUsers(Request $request)
    {

        return view('users.unapprovedUsers');
    }

    public function unapprovedUsersList(Request $request)
    {
        $users = user::where('betting', '=', 'no')->where('role', '=', 'client')->orderBy('id', 'asc')->get();
        return Datatables::of($users)
            ->addIndexColumn()
            ->editColumn('username', function ($row) {
                $link = '<a href="' . url('/') . '/admin/' . $row['id'] . '/userDetails" style="color:blue;">' . $row['username'] . '</a>';
                return $link;
            })
            ->editColumn('created_at', function ($row) {

                return date('Y-m-d', strtotime($row['created_at']));
            })
            ->editColumn('status', function ($row) {
                if ($row->status == 'active') {
                    $status = '<a role="button" class="activeDeactive" id="success-' . $row['id'] . '-users-id-status"><span class="badge badge-pill badge-soft-success font-size-12">Yes</span></a>';
                } else {
                    $status = '<a role="button" class="activeDeactive" id="danger-' . $row['id'] . '-users-id-status"><span class="badge badge-pill badge-soft-danger font-size-12">No</span></a>';
                }
                return $status;
            })
            ->editColumn('betting', function ($row) {
                if ($row->betting == 'yes') {
                    $status = '<a role="button" class="activeDeactive" id="success-' . $row['id'] . '-users-id-betting"><span class="badge badge-pill badge-soft-success font-size-12">Yes</span></a>';
                } else {
                    $status = '<a role="button" class="activeDeactive" id="danger-' . $row['id'] . '-users-id-betting"><span class="badge badge-pill badge-soft-danger font-size-12">No</span></a>';
                }
                return $status;
            })
            ->editColumn('transfer', function ($row) {
                if ($row->transfer == 'yes') {
                    $status = '<a role="button" class="activeDeactive" id="success-' . $row['id'] . '-users-id-transfer"><span class="badge badge-pill badge-soft-success font-size-12">Yes</span></a>';
                } else {
                    $status = '<a role="button" class="activeDeactive" id="danger-' . $row['id'] . '-users-id-transfer"><span class="badge badge-pill badge-soft-danger font-size-12">No</span></a>';
                }
                return $status;
            })
            ->addColumn('view_details', function ($row) {
                $btn = '<a href="' . url('/') . '/admin/' . $row['id'] . '/userDetails" class="mr-3 text-primary" data-toggle="tooltip" data-placement="top" title="" data-original-title="View Details" aria-describedby="tooltip224897"><i class="mdi mdi-eye font-size-18"></i></a>';


                return $btn;
            })
            ->rawColumns(['#', 'view_details', 'username', 'status', 'betting', 'transfer'])
            ->make(true);
    }

    public function userDetails($id)
    {
        $user = User::find($id);
        $withdraw = Withdraw::where('user_id', '=', $id)->orderBy('id', 'desc')->get();
        return view('users.userDetails', compact('user', 'withdraw'));
    }

    public function subAdmin(Request $request)
    {

        return view('users.subAdmin');
    }

    public function subAdminList(Request $request)
    {

        $users = user::where('id', '!=', '1')->where('role', '=', 'admin')->orderBy('id', 'asc')->get();
        return Datatables::of($users)
            ->addIndexColumn()
            ->editColumn('created_at', function ($row) {

                return date('Y-m-d', strtotime($row['created_at']));
            })
            ->editColumn('display_status', function ($row) {
                if ($row->status == 'active') {
                    $status = '<div id="status_show' . $row['id'] . '"><badge class="badge badge-success">Active</badge></div>';
                } else {
                    $status = '<div id="status_show' . $row['id'] . '"><badge class="badge badge-danger">Inactive</badge></div>';
                }
                return $status;
            })
            ->rawColumns(['#', 'display_status'])
            ->make(true);
    }

    public function allTransactionTable(Request $request)
    {

        $transaction = TransactionHistory::where('user_id', '=', $request->user_id)->orderBy('id', 'asc')->get();
        return Datatables::of($transaction)
            ->addIndexColumn()
            ->editColumn('amount', function ($row) {

                return ($row->type == 'credit') ? '+ ' . $row->amount . ' ₹ ' : '- ' . $row->amount . ' ₹ ';
            })
            ->rawColumns(['#', 'amount'])
            ->make(true);
    }

    public function creditTransactionTable(Request $request)
    {

        $transaction = TransactionHistory::where('user_id', '=', $request->user_id)->where('type', '=', 'credit')->orderBy('id', 'asc')->get();
        return Datatables::of($transaction)
            ->addIndexColumn()
            ->editColumn('amount', function ($row) {

                return '+ ' . $row->amount . ' ₹';
            })
            ->rawColumns(['#', 'amount'])
            ->make(true);
    }

    public function debitTransactionTable(Request $request)
    {

        $transaction = TransactionHistory::where('user_id', '=', $request->user_id)->where('type', '=', 'debit')->orderBy('id', 'asc')->get();
        return Datatables::of($transaction)
            ->addIndexColumn()
            ->editColumn('amount', function ($row) {

                return '- ' . $row->amount . ' ₹';
            })
            ->rawColumns(['#', 'amount'])
            ->make(true);
    }

    public function winningTable(Request $request)
    {

        $bid['getResultHistory'] = UserBid::select(DB::raw('users_bid.*,games.game_name'))
            ->leftJoin('games', 'users_bid.game_id', '=', 'games.id')->where('users_bid.user_id', '=', $request->user_id)->where('users_bid.is_win', '=', '1')->where('users_bid.bid_date', '=', $request->result_date)->orderBy('users_bid.id', 'desc')->get();
        echo json_encode($bid);
    }

    public function userBidHistoryTable(Request $request)
    {

        $bid = UserBid::select(DB::raw('users_bid.*,games.game_name,game_type.type'))
            ->leftJoin('games', 'users_bid.game_id', '=', 'games.id')
            ->leftJoin('game_type', 'users_bid.game_type_id', '=', 'game_type.id')
            ->where('users_bid.user_id', '=', $request->user_id)->orderBy('users_bid.id', 'asc')->get();
        return Datatables::of($bid)
            ->addIndexColumn()
            ->editColumn('open_digit', function ($row) {

                return ($row->open_digit == '') ? 'N/A' : $row->open_digit;
            })
            ->editColumn('close_digit', function ($row) {

                return ($row->close_digit == '') ? 'N/A' : $row->close_digit;
            })
            ->editColumn('open_pana', function ($row) {

                return ($row->open_pana == '') ? 'N/A' : $row->open_pana;
            })
            ->editColumn('close_pana', function ($row) {

                return ($row->close_pana == '') ? 'N/A' : $row->close_pana;
            })
            ->editColumn('created_at', function ($row) {

                return date('Y-m-d h:i A', strtotime($row['created_at']));
            })
            ->rawColumns(['#', 'close_pana', 'open_pana', 'open_digit', 'close_digit', 'created_at'])
            ->make(true);
    }

    public function addSubAdmin(Request $request)
    {

        $adminRole = Role::where('name',User::ROLE_ADMIN)->first();
        //dd($adminRole);
        $subadmin = User::create([

            'name' => $request->sub_admin_name,
            'username' => $request->user_name,
            'email' => $request->email,
            'password' => bcrypt($request->password),
        ]);
        $subadmin->role = 'admin';
        $subadmin->save();
        $subadmin->assignRole($adminRole);
        return response()->json(['msg' => '<p class="alert alert-success">SubAdmin Added Successfully.</p>', 'status' => 'success']);
    }

    public function sendNotification(Request $request)
    {

        $user = User::where('role', '=', 'client')->get();
        //dd($user);
        return view('notice.sendNotification', compact('user'));
    }

    public function userSendNotification(Request $request)
    {
        if ($request->user_id == 'all') {
            $users = User::where('role', '=', 'client')->get();
            foreach ($users as $key => $value) {

                $noti = $this->fcm($request->notice_title, $request->notification_content, $value->device_token);
            }
        } else {

            $user = User::find($request->user_id);
            $noti = $this->fcm($request->notice_title, $request->notification_content, $user->device_token);

        }
        return response()->json(['msg' => '<p class="alert alert-success">Notification Sended Successfully.</p>', 'status' => 'success']);
    }

    public function noticeManagement(Request $request)
    {

        return view('notice.noticeManagement');
    }

    public function noticeList(Request $request)
    {

        $notice = Notice::all();
        return Datatables::of($notice)
            ->addIndexColumn()
            ->editColumn('display_status', function ($row) {
                if ($row->status == 'active') {
                    $status = '<div id="status_show' . $row['id'] . '"><badge class="badge badge-success">Active</badge></div>';
                } else {
                    $status = '<div id="status_show' . $row['id'] . '"><badge class="badge badge-danger">Inactive</badge></div>';
                }
                return $status;
            })
            ->rawColumns(['#', 'display_status'])
            ->make(true);
    }

    public function addNotice(Request $request)
    {

        $notice = Notice::create([

            'title' => $request->notice_title,
            'content' => $request->description,
            'date' => date('Y-m-d', strtotime($request->notice_date)),
            'status' => 'active',
        ]);
        $notice->save();
        return response()->json(['msg' => '<p class="alert alert-success">Notice Added Successfully.</p>', 'status' => 'success']);
    }

    public function switchStatus(Request $request)
    {
        //dd($request);

        if ($request->status_name == 'status') {
            DB::statement('update ' . $request->table . ' set ' . $request->status_name . '=IF(' . $request->status_name . '="active","blocked","active") where id=' . $request->id);
            DB::table('oauth_access_tokens')
                ->whereIn('user_id', [$request->id])
                ->update(['revoked' => true]);
            if($request->table == 'users'){    
                $user = User::select('id','status')->find($request->id);
                $status = $user->status;
                $userid = $user->id;
                broadcast(new UserBlock($status,$userid));
            }
            if($request->table == 'games'){
                broadcast(new RefreshGame($request->id));
            }

            if($request->table == 'starline_games'){
                broadcast(new RefreshStarlineGame($request->id));
            }
                    
        } else {
            DB::statement('update ' . $request->table . ' set ' . $request->status_name . '=IF(' . $request->status_name . '="no","yes","no") where id=' . $request->id);

            if($request->status_name == 'betting'){
                $user = User::select('id','betting')->find($request->id);
                $status = $user->betting;
                $userid = $user->id;
                
                broadcast(new UserBattingOff($status,$userid));
            }
        }
        $user = User::find($request->id);
        if (!empty($user)) {
            $this->fcm('refresh', '', $user->device_token);
        }

    }

    public function openCloseMarket(Request $request)
    {
        DB::statement('update ' . $request->table . ' set ' . $request->status_name . '=IF(' . $request->status_name . '="blocked","active","blocked") where id=' . $request->id);

        if($request->table =="starline_games"){
            broadcast(new RefreshStarlineGame($request->id));
        }
    }

    public function changeSecurityPin(Request $request)
    {
        $user = User::find($request->user_id);
        $user->security_pin = $request->security_pin;
        $user->save();

        return response()->json(['msg' => '<p class="alert alert-success">Security Pin changed successfully!.</p>', 'status' => 'success', 'security_pin' => $user->security_pin]);
    }

    public function submitChangePassword(Request $request)
    {
        $profile = User::find(\Auth::user()->id);
        if (Hash::check($request->oldpass, $profile->password)) {

            $profile->password = bcrypt($request->newpass);
            $profile->password_string = $request->newpass;
            $profile->save();
            return response()->json(['status' => 'success', 'msg' => 'Password updated successfully.']);
        } else {
            return response()->json(['status' => 'fail', 'msg' => '<p class="alert alert-danger">Old password does not matched.</p>']);

        }
    }

    public function slideImage(Request $request)
    {
        return view('others.slideImage');
    }

    public function sliderImagesList(Request $request)
    {
        $users = ImageSlide::orderBy('id', 'asc')->get();
        return Datatables::of($users)
            ->addIndexColumn()
            ->editColumn('slider_image', function ($row) {
                $link = '<img src="' . $row['image'] . '" height="100" width="100"></img>';
                return $link;
            })
            ->editColumn('created_at', function ($row) {

                return date('Y-m-d', strtotime($row['created_at']));
            })
            ->addColumn('display_status', function ($row) {
                if ($row->status == 'active') {
                    $status = '<a role="button" ><span class="badge badge-pill badge-soft-success font-size-12">Yes</span></a>';
                } else {
                    $status = '<a role="button" class="activeDeactive" id="danger-' . $row['id'] . '-users-id-status"><span class="badge badge-pill badge-soft-danger font-size-12">No</span></a>';
                }
                return $status;
            })
            ->rawColumns(['#', 'slider_image', 'display_status', 'created_at'])
            ->make(true);
    }

    public function userQuery(Request $request)
    {
        return view('others.userQuery');
    }

    public function userQueryList(Request $request)
    {
        $users = UserInquiry::orderBy('id', 'asc')->get();
        return Datatables::of($users)
            ->addIndexColumn()
            ->editColumn('insert_date', function ($row) {

                return date('Y-m-d h:i:s', strtotime($row['created_at']));
            })
            ->rawColumns(['#', 'insert_date'])
            ->make(true);
    }

    public function addSliderImage(Request $request)
    {
        $image_order = $request->display_order;
        if ($request->hasfile('file')) {
            $file = $request->file('file');
            $actual_name = $request->file('file')->getClientOriginalName();
            $extension = $request->file('file')->extension();
            $filename = time() . '_' . $actual_name;
            $file->move(public_path() . '/asset/images/', $filename);

            $pic = url('/') . '/asset/images/' . $filename;
        }

        ImageSlide::create([
            'image' => $pic,
            'image_order' => $image_order,
            'status' => '1',
        ]);

        return response()->json(['msg' => '<p class="alert alert-success">Slider image added successfully.</p>', 'status' => 'success']);

    }

    public function deleteImage(Request $request)
    {

        $slider = ImageSlide::find($request->id);
        $slider->delete();
        return response()->json(['msg' => '<p class="alert alert-success">Slider image deleted successfully.</p>', 'status' => 'success']);

    }

    public function howToPlay(Request $request)
    {

        $text = HowtoPlay::find(1);
        //dd($rates);
        return view('others.howToPay', compact('text'));
    }

    public function addHowtoPlay(Request $request)
    {
        $text = HowtoPlay::find(1);
        $text->text = $request->description;
        $text->link = $request->video_link;
        $text->save();
        return response()->json(['msg' => '<p class="alert alert-success">Updated successfully!</p>', 'status' => 'success']);
    }
}
