<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use App\Models\User;
use App\Models\TransactionHistory;
use App\Models\Withdraw;
use App\Events\UpdateBalance;
use Yajra\DataTables\DataTables;
use DB;

class WalletController extends Controller
{
    public function addFund(Request $request)
    {

        $users = User::where('role', '=', 'client')->get();
        return view('wallet.addFund', compact('users'));
    }

    public function userAddFund(Request $request)
    {

        $user = User::find($request->user_id);
        //dd($user->available_balance);
        $user->available_balance = $user->available_balance + $request->user_amount;
        $user->save();

        broadcast(new UpdateBalance($user->available_balance,$user->id));

        $fund = TransactionHistory::create([

            'admin_id' => 0,
            'user_id' => $user->id,
            'amount' => $request->user_amount,
            'transaction_note' => 'Added from admin portal',
            'transfer_note' => '',
            'type' => 'credit',
            'date' => date('Y-m-d H:i:s'),
            'tx_req_number' => random_int(100000, 999999),

        ]);
        $fund->save();
        $this->fcm('refresh', '', $user->device_token);
        //dd($test);
        return response()->json(['msg' => '<p class="alert alert-success">Fund Added Successfully.</p>', 'status' => 'success']);
    }

    public function withdrawBalanceUser(Request $request)
    {

        $user = User::find($request->user_id);
        if ($request->amount > $user->available_balance) {

            return response()->json(['msg' => '<p class="alert alert-danger">Withdraw amount can not be greater than balance amount.</p>', 'status' => 'fail']);
        }
        $user->available_balance = $user->available_balance - $request->amount;
        $user->save();

        broadcast(new UpdateBalance($user->available_balance,$user->id));

        $fund = TransactionHistory::create([

            'admin_id' => 0,
            'user_id' => $user->id,
            'amount' => $request->amount,
            'transaction_note' => 'Amount Withdraw By Admin',
            'transfer_note' => '',
            'type' => 'debit',
            'date' => date('Y-m-d H:i:s'),
            'tx_req_number' => random_int(100000, 999999),

        ]);
        $fund->save();
        $this->fcm('refresh', '', $user->device_token);

        return response()->json(['msg' => '<p class="alert alert-success">Withdraw Successfully.</p>', 'status' => 'success']);
    }

    public function withdrawRequestList(Request $request)
    {

        $users = Withdraw::select(DB::raw('withdraw.*,users.username,users.phone_number'))
            ->leftJoin('users', 'withdraw.user_id', '=', 'users.id')->orderBy('withdraw.id', 'asc')->get();
        return Datatables::of($users)
            ->addIndexColumn()
            ->editColumn('username', function ($row) {
                $link = '<a href="' . url('/') . '/admin/' . $row['user_id'] . '/userDetails" style="color:blue;">' . $row['username'] . '</a>';
                return $link;
            })
            ->editColumn('status', function ($row) {
                if ($row->status == '0') {
                    $status = '<badge class="badge badge-pill badge-soft-danger font-size-12">Rejected</badge>';
                } else if ($row->status == '1') {
                    $status = '<badge class="badge badge-pill badge-soft-success font-size-12">Accepted</badge>';
                } else {
                    $status = '<badge class="badge badge-pill badge-soft-info font-size-12">Pending</badge>';
                }
                return $status;
            })
            ->addColumn('view', function ($row) {

                $btn = '<a role="button" data-href="' . url('/') . '/admin/' . $row['id'] . '/withdrawDetails" class="mr-3 text-primary openViewWithdrawRequest" data-toggle="tooltip" data-placement="top" title="" data-original-title="View Details"><i class="mdi mdi-eye font-size-18"></i></a>';


                return $btn;
            })
            ->addColumn('action', function ($row) {

                if ($row->status == '3') {
                    $btn = '<td><button class="btn btn-primary btn-sm w-xs mr-1" id="accept" data-withdraw_request_id="' . $row['id'] . '" data-user_name="' . $row['username'] . '" data-mobile="' . $row['phone_number'] . '" data-request_amount="' . $row['amount'] . '" data-request_number="' . $row['request_number'] . '">Approve</button><button class="btn btn-danger btn-sm w-xs" id="reject" data-withdraw_request_id="' . $row['id'] . '">Reject</button></td>';

                } else {
                    $btn = '<button class="btn btn-primary btn-sm w-xs mr-1" data-withdraw_request_id="' . $row['id'] . '" disabled="">Approve</button><button class="btn btn-danger btn-sm w-xs" data-withdraw_request_id="' . $row['id'] . '" disabled="">Reject</button>';
                }


                return $btn;
            })
            ->rawColumns(['#', 'view', 'username', 'status', 'action'])
            ->make(true);
    }

    public function withdrawDetails($id)
    {

        $withdraw = Withdraw::select(DB::raw('withdraw.*,users.username,users.phone_number'))
            ->leftJoin('users', 'withdraw.user_id', '=', 'users.id')
            ->where('withdraw.id', '=', $id)->first();

        $view = \View::make('wallet.withdrawDetailsPartial', compact('withdraw'));
        $html = $view->render();
        return $html;
    }

    public function approveWithdrawRequest(Request $request)
    {

        $withdraw = Withdraw::find($request->withdraw_req_id);
        $withdraw->remark = $request->remark;
        $withdraw->status = '1';
        $withdraw->accept_date = date('Y-m-d H:i:s');

        if ($request->hasfile('file')) {
            $file = $request->file('file');
            $actual_name = $request->file('file')->getClientOriginalName();
            $extension = $request->file('file')->extension();
            $filename = time() . '_' . $actual_name;
            $file->move(public_path() . '/asset/images/', $filename);

            $withdraw->receipt = url('/') . '/asset/images/' . $filename;


        }
        $withdraw->save();

        $user = User::find($withdraw->user_id);

        $fund = TransactionHistory::create([

            'admin_id' => 0,
            'user_id' => $withdraw->user_id,
            'amount' => $withdraw->amount,
            'transaction_note' => 'Withdrawal Approved',
            'transfer_note' => '',
            'type' => 'debit',
            'date' => date('Y-m-d H:i:s'),
            'tx_req_number' => $withdraw->request_number,

        ]);
        $fund->save();
        $this->fcm('refresh', '', $user->device_token);
        return response()->json(['msg' => '<p class="alert alert-success">Withdraw request accepted Successfully.</p>', 'status' => 'success']);
    }

    public function rejectWithdrawRequest(Request $request)
    {

        $withdraw = Withdraw::find($request->withdraw_req_id);
        $withdraw->remark = $request->remark;
        $withdraw->status = '0';
        $withdraw->save();

        $fund = TransactionHistory::create([

            'admin_id' => 0,
            'user_id' => $withdraw->user_id,
            'amount' => $withdraw->amount,
            'transaction_note' => 'Withdrawal Rejected',
            'transfer_note' => $request->remark,
            'type' => 'credit',
            'date' => date('Y-m-d H:i:s'),
            'tx_req_number' => $withdraw->request_number,

        ]);
        $fund->save();

        $user = User::find($withdraw->user_id);
        $user->available_balance = $user->available_balance + $withdraw->amount;
        $user->save();
        broadcast(new UpdateBalance($user->available_balance,$user->id));
        //$this->fcm('refresh', '', $user->device_token);
        return response()->json(['msg' => '<p class="alert alert-success">Withdraw request rejected Successfully.</p>', 'status' => 'success']);
    }

    public function acceptAutoFund(Request $request)
    {

        $fund = TransactionHistory::find($request->id);
        $fund->status = '1';
        $fund->save();

        $user = User::find($fund->user_id);
        $user->available_balance = $user->available_balance + $fund->amount;
        $user->save();
        $this->fcm('refresh', '', $user->device_token);
        return response()->json(['msg' => '<p class="alert alert-success">Fund Approved Successfully.</p>', 'status' => 'success']);
    }

    public function rejectAutoFund(Request $request)
    {
        $fund = TransactionHistory::find($request->id);
        $fund->status = '0';
        $fund->reject_reason = $request->reject_auto_remark;
        $fund->save();
        return response()->json(['msg' => '<p class="alert alert-success">Fund Rejected Successfully.</p>', 'status' => 'success']);
    }

    public function deleteAutoFund(Request $request)
    {
        $fund = TransactionHistory::find($request->id);
        $fund->delete();
        return response()->json(['msg' => '<p class="alert alert-success">Fund deleted Successfully.</p>', 'status' => 'success']);
    }

}
