<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Api\BaseController;
use Spatie\Permission\Models\Role;
use App\Models\User;
use App\Models\MainSetting;
use Validator;
use Auth;
use Hash;

class AuthController extends BaseController
{

    public function login(Request $request){
        $json = array();
        $validator = Validator::make($request->all(), [
            'phone_number'     => 'required',
            'password'  => 'required|min:4|max:8',
            
        ]);

        if($validator->fails()){
            $errors = $validator->errors()->toArray();
            $message = "";
            foreach($errors as $key => $error){
                $message .= $error[0]."\n";
            }
            return $this->sendError($message,[]);
        }

        $checkUser = User::where('phone_number',$request->phone_number)
            ->where('role', User::ROLE_CLIENT)->first();

        if(!$checkUser){
            
            return $this->sendError('Phone number does not exit!',[],200);
        }

        if (!Hash::check($request->password, $checkUser->password)) {
            return $this->sendError('Invalid password!',[],200);
        }

        if($checkUser->status != 'active'){
            return $this->sendError('Account is blocked,Please contact admin.',[],200);
        }

        $credentials = array(
            'phone_number'     => $request->phone_number,
            'password'  => $request->password,
            'role'      => User::ROLE_CLIENT,
            'status'    => 'active',
        );

        if(Auth::attempt($credentials)){
            $user = Auth::user();
            /*$user->device_token = $request->device_token;
            $user->device_id = $request->device_id;*/
            $user->save();
            $user->makeHidden(['password','deleted_at','created_at','updated_at','device_id','device_token','email_verified_at','security_pin']);

            $setting=MainSetting::first();
            $json['userDetails'] = $user->toArray();
    	    //$json['userDetails']['token'] = $user->createToken(\Config::get('app.name'))->accessToken;
            $json['userDetails']['token'] = $user->createToken('user')->plainTextToken;
    	    $json['userDetails']['login_msg'] = str_replace("\r\n"," ",$setting->login_msg);
            $json['userDetails']['global_on'] = ($setting->global_betting == '1' ? 'yes' : 'no');

            return $this->sendResponse($json,'Login Success!');
        } else {
            return $this->sendError('invalid login details!',[]);
        }
    }

    public function logout(Request $request){
        Auth::logout();
        return $this->sendResponse([],'logout User Success!');
    }
}
