<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Api\BaseController;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\UserBid;
use App\Models\Games;
use App\Models\GameType;
use App\Models\DeclareResult;
use App\Models\StarlineUserBid;
use App\Models\StarlineGames;
use App\Models\TransactionHistory;
use App\Models\MainSetting;
use App\Models\StarlineDeclareResult;
use App\Services\CommonService;
use App\Traits\CommonTraits;
use App\Events\UpdateBalance;
use Validator;
use Session;
use Auth;
use DB;
use Carbon\Carbon;

class BidController extends BaseController
{
    use CommonTraits;

    protected $commonservice;
    public function __construct(CommonService $commonservice){
        $this->commonservice = $commonservice;
    }

    public function createBid(Request $request)
    {
        $bid_list = $request->bids;
        //dd($request->bids,$bid_list);
        $user = User::find(\Auth::user()->id);
        $main = MainSetting::find(1);
        $game = Games::find($request->game_id);
        $market_time = $this->getMarketTime($game->market_time);

        //if(Carbon::parse($market_time->close_time)->format('H:i')  )
        //$request->game_id

        

        if($request->total_point > $main->max_bid_amount){
            return $this->sendError('Your Bid amount are Greater than Max Bid', []);
        }

        if($request->total_point > $user->available_balance){
            return $this->sendError('Insufficient Wallet Balance', []);
        }

        foreach ($bid_list as $key => $value) {

            if ($request->game_type_id != '2') {

                $session = (isset($value['session'])) ? $value['session'] : 'open';

                if($session == 'open'){
                    if(Carbon::parse($market_time->open_time)->lt(Carbon::now())){
                        return $this->sendError('Game is closed', ["time_error" => true]);
                    }

                } else {
                    if(Carbon::parse($market_time->close_time)->lt(Carbon::now())){
                        return $this->sendError('Game is closed', ["time_error" => true]);
                    }
                }
            } else {
                if(Carbon::parse($market_time->close_time)->lt(Carbon::now())){
                    return $this->sendError('Game is closed', ["time_error" => true]);
                }
                $session = "none";
            }
            

            if ($request->game_type_id == '2') {

                //$jodi = str_split($value->open_digit);
                $open_digit = $value['digit'];
                $close_digit = 0;
                $open_pana = '';
                $close_pana = '';

            } else if ($request->game_type_id == '7') { //full sangum

                $open_digit = $this->sum($value['open_pana']);
                $close_digit = $this->sum($value['close_pana']);
                $open_pana = $value['open_pana'];
                $close_pana = $value['close_pana'];

            } else if ($request->game_type_id == '6' ) {

                $open_digit = 0;
                $close_digit = 0;
                $open_pana = $value['open_pana'];
                $close_pana = $value['close_pana'];

            } else if ($request->game_type_id == '1' || $request->game_type_id == '3' || $request->game_type_id == '4' || $request->game_type_id == '5') {

                $open_digit = $value['digit'];
                $close_digit = 0;
                $open_pana = '';
                $close_pana = '';

            }

            

            //dd($session);

            $tx_number = random_int(100000, 999999);
            $bid = UserBid::create([

                'user_id' => \Auth::user()->id,
                'game_id' => $request->game_id,
                'tx_id' => $tx_number,
                'game_type_id' => $request->game_type_id,
                'session' => $session,
                'open_pana' => $open_pana,
                'open_digit' => $open_digit,
                'close_pana' => $close_pana,
                'close_digit' => $close_digit,
                'points' => $value['points'],
                'result_declare' => '0',
                'is_win' => '0',
                'wining_amount' => 0,
                'bid_date' => date('Y-m-d')

            ]);
            

            $game = Games::find($request->game_id);
            $gametype = GameType::find($request->game_type_id);
            $fund = TransactionHistory::create([

                'admin_id' => 0,
                'user_id' => \Auth::user()->id,
                'amount' => $value['points'],
                'transaction_note' => 'Bid Placed For ' . $game->game_name . '(' . $gametype->type . ')',
                'transfer_note' => '',
                'type' => 'debit',
                'date' => date('Y-m-d H:i:s'),
                'tx_req_number' => $tx_number,

            ]);
            

            
            $user->available_balance = $user->available_balance - $value['points'];
            $user->save();

            broadcast(new UpdateBalance($user->available_balance,$user->id));

        }

        return $this->sendResponse(["wallet" => $user->available_balance], 'Bid Created Succesfully!');
    }

    public function bidHistory(Request $request)
    {

        $bid = $this->commonservice->getBidHistory($request);

        if (!$bid) {
            return $this->sendResponse([], 'bid history list!');
        }

        return $this->sendResponse($bid, 'bid history list!');

    }

    public function winHistory(Request $request)
    {

        $win = $this->commonservice->getWinHistory($request);
        

        if (!$win) {
            return $this->sendResponse([], 'bid history list!');
        }

        return $this->sendResponse($win, 'bid history list!');
    }

    public function walletHistory(Request $request)
    {

        $wallet = $this->commonservice->getWalletHistory($request);
        if (!$wallet) {
            return $this->sendResponse([], 'wallet history list!');
        }
        return $this->sendResponse($wallet, 'wallet history list!');
    }

    public function resultChart(Request $request)
    {
        $week = $this->getLastTenWeek();
        $partnerNumber = array("16", "61", "27", "72", "38", "83", "49", "94", "50", "05");
        $color = '#000000';
        foreach($week as $key => $ww){
            $resultArray = [];
            $dates = $this->getWeekDates($ww['week_start'],$ww['week_end']);
            foreach($dates as $l => $d){

                if(Carbon::parse($d)->format('Y-m-d') > Carbon::parse('now')->format('Y-m-d')){
                    continue;
                }
                
                $result = DeclareResult::where('game_id', '=', $request->game_id)->whereDate('declare_date',Carbon::parse($d)->format('Y-m-d'))->first();
                if($result){
                    $leftside = ($result->open_number == '') ? '***' : $result->open_number;
                    $centerOne = ($result->open_result == '') ? '*' : $result->open_result;
                    $centerTwo = ($result->close_result == '') ? '*' : $result->close_result;
                    $rightside = ($result->close_number == '') ? '***' : $result->close_number;
                    if ($centerOne == $centerTwo && $centerOne != '*') {
                        $color = '#FF0000';
                    } else if (in_array($centerOne . $centerTwo, $partnerNumber) || in_array($centerTwo . $centerOne, $partnerNumber)) {
                        $color = '#FF0000';
                    }
                    array_push($resultArray, array('date' => date('D (d M Y)', strtotime($result->declare_date)), 'left' => $leftside, 'center' => $centerOne . $centerTwo, 'right' => $rightside, 'color' => $color));
                } else {
                    $leftside = '***';
                    $centerOne = '*';
                    $centerTwo = '*';
                    $rightside = '***';                    
                    $color = '#000000';                    
                    array_push($resultArray, array('date' => date('D (d M Y)', strtotime($d)), 'left' => $leftside, 'center' => $centerOne . $centerTwo, 'right' => $rightside, 'color' => $color));
                }

            }
            $week[$key]['data'] = $resultArray;

        }
        //dd($week);
        /*$result = DeclareResult::where('game_id', '=', $request->game_id)->orderBy('id', 'asc')->get();
        $resultArray = [];
        $partnerNumber = array("16", "61", "27", "72", "38", "83", "49", "94", "50", "05");
        $color = '#000000';
        foreach ($result as $key => $value) {

            $leftside = ($value->open_number == '') ? '***' : $value->open_number;
            $centerOne = ($value->open_result == '') ? '*' : $value->open_result;
            $centerTwo = ($value->close_result == '') ? '*' : $value->close_result;
            $rightside = ($value->close_number == '') ? '***' : $value->close_number;
            if ($centerOne == $centerTwo) {

                $color = '#FF0000';
            } else if (in_array($centerOne . $centerTwo, $partnerNumber) || in_array($centerTwo . $centerOne, $partnerNumber)) {
                $color = '#FF0000';
            }
            array_push($resultArray, array('date' => date('D (d M Y)', strtotime($value->declare_date)), 'left' => $leftside, 'center' => $centerOne . $centerTwo, 'right' => $rightside, 'color' => $color));

        }*/
        $data['result'] = $week;
        $data['game'] = Games::select('game_name','game_name_hindi')->find($request->game_id);
        return $this->sendResponse($data, 'result chart');
    }

    public function starlineResultChart2(Request $request)
    {

        $result = StarlineDeclareResult::select('declare_date')->groupBy('declare_date')->get();
        //dd($result);
        $games = StarlineGames::select('id','game_name')->where('status', '=', 'active')->orderBy('open_time', 'asc')->get();
        $results= [];
        foreach ($result as $key => $value) {
            $dayResult = [];
            foreach ($games as $gkey => $gvalue) {

                $gameresult = StarlineDeclareResult::where('declare_date', '=', $value->declare_date)->where('game_id', '=', $gvalue->id)->first();
                if (!empty($gameresult)) {
                    $leftside = ($gameresult->open_number == '') ? '***' : $gameresult->open_number;
                    $centerOne = ($gameresult->open_result == '') ? '*' : $gameresult->open_result;
                    array_push($dayResult, array("game_name" => $gvalue->game_name, "digit" => $centerOne, "pana" => $leftside));
                } else {
                    array_push($dayResult, array("game_name" => $gvalue->game_name, "digit" => "*", "pana" => "***"));
                }

            }
            array_push($results, array("date" => $value->declare_date, "chart" => $dayResult));

        }

        //dd($chart);
        $data['result']=$results;
        $data['game']=$games;
        /*$response = array();
        $response['error'] = false;
        $response['game_list'] = $games;
        $response['data'] = (empty($results)) ? (object)[] : $results;
        $response['message'] = 'Result chart';*/
        return $this->sendResponse($data, 'starline result chart');
        
    }

    public function starlineResultChart(Request $request)
    {

        $result = StarlineDeclareResult::where('game_id', '=', $request->game_id)->orderBy('id', 'asc')->get();
        $dayResult = [];
        foreach ($result as $gkey => $gvalue) {
            
            if (!empty($gvalue)) {
                $leftside = ($gvalue->open_number == '') ? '***' : $gvalue->open_number;
                $centerOne = ($gvalue->open_result == '') ? '*' : $gvalue->open_result;
                array_push($dayResult, array("game_name" => $gvalue->game_name, "digit" => $centerOne, "pana" => $leftside));
            } else {
                array_push($dayResult, array("game_name" => $gvalue->game_name, "digit" => "*", "pana" => "***"));
            }

        }

        $week = $this->getLastTenWeek();
        $partnerNumber = array("16", "61", "27", "72", "38", "83", "49", "94", "50", "05");
        $color = '#000000';
        foreach($week as $key => $ww){
            $resultArray = [];
            $dates = $this->getWeekDates($ww['week_start'],$ww['week_end']);
            foreach($dates as $l => $d){

                if(Carbon::parse($d)->format('Y-m-d') > Carbon::parse('now')->format('Y-m-d')){
                    continue;
                }
                $result = StarlineDeclareResult::where('game_id', '=', $request->game_id)->whereDate('declare_date',Carbon::parse($d)->format('Y-m-d'))->first();
                if($result){
                    $leftside = ($result->open_number == '') ? '***' : $result->open_number;
                    $centerOne = ($result->open_result == '') ? '*' : $result->open_result;
                    $centerTwo = ($result->close_result == '') ? '*' : $result->close_result;
                    //$rightside = ($result->close_number == '') ? '***' : $result->close_number;
                    if ($centerOne == $centerTwo && $centerOne != '*') {
                        $color = '#FF0000';
                    } else if (in_array($centerOne . $centerTwo, $partnerNumber) || in_array($centerTwo . $centerOne, $partnerNumber)) {
                        $color = '#FF0000';
                    }
                    array_push($resultArray, array('date' => date('D (d M Y)', strtotime($result->declare_date)), 'left' => $leftside, 'center' => $centerOne . $centerTwo, 'color' => $color));
                } else {
                    $leftside = '***';
                    $centerOne = '*';
                    $centerTwo = '*';
                    //$rightside = '***';                    
                    $color = '#000000';                    
                    array_push($resultArray, array('date' => date('D (d M Y)', strtotime($d)), 'left' => $leftside, 'center' => $centerOne . $centerTwo, 'color' => $color));
                }

            }
            $week[$key]['data'] = $resultArray;

        }

        $data['result'] = $week;
        $data['game'] = StarlineGames::select('game_name')->find($request->game_id);
        return $this->sendResponse($data, 'result chart');  
    }

    public function chart(Request $request)
    {
        $game = Games::find($request->game_id);
        $result = DeclareResult::where('game_id', '=', $request->game_id)->orderBy('id', 'asc')->get();
        $resultArray = [];
        $partnerNumber = array("16", "61", "27", "72", "38", "83", "49", "94", "50", "05");
        $color = '#000000';
        foreach ($result as $key => $value) {

            $leftside = ($value->open_number == '') ? '***' : $value->open_number;
            $centerOne = ($value->open_result == '') ? '*' : $value->open_result;
            $centerTwo = ($value->close_result == '') ? '*' : $value->close_result;
            $rightside = ($value->close_number == '') ? '***' : $value->close_number;
            if ($centerOne == $centerTwo) {

                $color = '#FF0000';
            } else if (in_array($centerOne . $centerTwo, $partnerNumber) || in_array($centerTwo . $centerOne, $partnerNumber)) {
                $color = '#FF0000';
            }
            array_push($resultArray, array('date' => date('D (d M Y)', strtotime($value->declare_date)), 'left' => $leftside, 'center' => $centerOne . $centerTwo, 'right' => $rightside, 'color' => $color));

        }
        $data['result'] = $resultArray;
        $data['game_name'] = $game->game_name;
        return $this->sendResponse($data, 'result chart');
    }

    public function starlineBidHistory(Request $request)
    {

        $bid = $this->commonservice->getStarlineBidHistory($request);
        if (!$bid) {
            return $this->sendResponse([], 'bid history list!');
        }
        return $this->sendResponse($bid, 'bid history list!');
    }

    public function starlineWinHistory(Request $request)
    {

        $win = $this->commonservice->getStarlineWinHistory($request);

        if (!$win) {
            return $this->sendResponse([], 'win history list!');
        }

        return $this->sendResponse($win, 'win history list!');
    }

    public function starlineCreateBid(Request $request)
    {
        $bid_list = $request->bids;
        $game = StarlineGames::find($request->game_id);

        $main = MainSetting::find(1);
        
        $user = User::find(\Auth::user()->id);
        if(Carbon::parse($game->open_time)->lt(Carbon::now())){
            return $this->sendError('Game is closed', ["time_error" => true]);
        }
       
        if($request->total_point > $main->max_bid_amount){
            return $this->sendError('Your Bid amount are Greater than Max Bid', []);
        }

        if($request->total_point > $user->available_balance){
            return $this->sendError('Insufficient Wallet Balance', []);
        }
       
        foreach ($bid_list as $key => $value) {      



            $open_digit = $value['digit'];                
            $open_pana = ''; 

            $tx_number = random_int(100000, 999999);
            $bid = StarlineUserBid::create([

                'user_id' => \Auth::user()->id,
                'game_id' => $request->game_id,
                'tx_id' => $tx_number,
                'game_type_id' => $request->game_type_id,
                'session' => 'none',
                'open_pana' => $open_pana,
                'open_digit' => $open_digit,
                'points' => $value['points'],
                'result_declare' => '0',
                'is_win' => '0',
                'wining_amount' => 0,
                'bid_date' => date('Y-m-d')

            ]);
            

            $game = StarlineGames::find($request->game_id);
            $gametype = GameType::find($request->game_type_id);
            $fund = TransactionHistory::create([

                'admin_id' => 0,
                'user_id' => \Auth::user()->id,
                'amount' => $value['points'],
                'transaction_note' => 'Bid Placed For ' . $game->game_name . '(' . $gametype->type . ')',
                'transfer_note' => '',
                'type' => 'debit',
                'date' => date('Y-m-d H:i:s'),
                'tx_req_number' => $tx_number,

            ]);            

            
            $user->available_balance = $user->available_balance - $value['points'];
            $user->save();

            broadcast(new UpdateBalance($user->available_balance,$user->id));

        }

        return $this->sendResponse(["wallet" => $user->available_balance], 'Bid Created Succesfully!');
    }

    public function sum($num)
    {
        $sum = 0;
        for ($i = 0; $i < strlen($num); $i++) {
            $sum += $num[$i];
        }
        return substr($sum, -1, 1);
    }
}
