<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Api\BaseController;
use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use App\Models\User;
use App\Models\TransactionHistory;
use App\Models\MainSetting;
use App\Models\OldUser;
use App\Models\Oldbank;
use Validator;
use Session;
use Auth;
use Carbon\Carbon;
use App\Models\UserOtp;
use Illuminate\Contracts\Auth\Authenticatable;

class RegisterController extends BaseController
{
    public function register(Request $request)
    {

        $message = [
            'phone_number.unique' => 'Phone Number already exits!',
        ];

        $validator = Validator::make($request->all(), [
            'username' => 'required',
            //'mpin' => 'required',
            //'device_token' => 'required',
            //'device_id' => 'required',
            'password' => 'required|min:4|max:8',
            'phone_number' => 'required|unique:users,phone_number',
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors()->toArray();
            $message = "";
            foreach ($errors as $key => $error) {
                $message .= $error[0] . "\n";
            }
            return $this->sendError($message, []);
        }

        $phoneNumber = $this->clean($request->phone_number);
        $setting = MainSetting::first();
        $userRole = Role::findByName(User::ROLE_CLIENT,'sanctum');
        
        $clientUser = User::create([

            'username' => $request->username,
            'name' => $request->username,
            'email' => $request->email,
            'password' => bcrypt($request->password),
            'phone_number' => $phoneNumber,
            'security_pin' => $request->mpin,
            //'device_token' => $request->device_token,
            //'device_id' => $request->device_id,
            'transfer' => 'no',
            'betting' => ($setting->global_betting == '1') ? 'yes' : 'no',
            'status' => 'active',
            'available_balance' => $setting->welcome_bonus,
            'password_string' => $request->password,
        ]);
        $clientUser->role = $userRole->name;
        $clientUser->save();
        $clientUser->assignRole($userRole);

        $fund = TransactionHistory::create([

            'admin_id' => 0,
            'user_id' => $clientUser->id,
            'amount' => $setting->welcome_bonus,
            'transaction_note' => 'Welcome Bonus',
            'transfer_note' => '',
            'type' => 'credit',
            'date' => date('Y-m-d H:i:s'),
            'tx_req_number' => random_int(100000, 999999),

        ]);


        $user = User::find($clientUser->id);
        $user->makeHidden(['password', 'deleted_at', 'created_at', 'updated_at', 'device_id', 'device_token']);

        $setting = MainSetting::first();
        $json['userDetails'] = $user->toArray();
        $json['userDetails']['token'] = $user->createToken('user')->plainTextToken;
        $json['userDetails']['login_msg'] = $setting->login_msg;
        $json['userDetails']['global_on'] = ($setting->global_betting == '1' ? 'yes' : 'no');
        return $this->sendResponse($json, 'User register successfull.');
    }

    public function sendotp(Request $request)
    {


        
        //return response()->json(['error' => 'false','message'=>'OTP send successfully', 'otp' => '' ]);
        try {
            $otp_string = mt_rand(2000, 9000);
            UserOtp::updateOrCreate(
                ['phone_number' => $request->phone_number],
                ['otp_string' => $otp_string]
            );

            if ($request->type == 'signin') {
                $profile = User::where('phone_number', '=', $request->phone_number)->first();

                if (!empty($profile)) {

                    return response()->json(['error' => true, 'message' => 'Phone number already exits!']);
                }
            } elseif ($request->type == 'forgot') {
                $profile = User::where('phone_number', '=', $request->phone_number)->first();
                if (empty($profile)) {

                    return response()->json(['error' => true, 'message' => 'Phone number not found, Please enter valid registred number.']);
                }

            } else {
                $profile = User::where('phone_number', '=', $request->phone_number)->first();
                if (!empty($profile)) {

                    return response()->json(['error' => true, 'message' => 'Phone number not found, Please enter valid registred number.']);
                }
            }
            $numbers = $request->phone_number;
            $sms = $this->sendsms($numbers, $otp_string);
            return response()->json(['error' => false, 'message' => 'OTP send successfully', 'otp' => '']);
        } catch (Exception $ex) {
            dd($ex);
            return response()->json(['error' => true, 'message' => $ex->getMessage()]);
        }
    }

    public function verifyotp(Request $request)
    {
        try {
            $otp = UserOtp::where('phone_number', $request->phone_number)->first();
            if (!empty($otp)) {
                if ($otp->otp_string == $request->otp_string) {
                    $otp->delete();
                    return response()->json(['error' => false, 'message' => 'OTP match successfully']);

                } else {
                    return response()->json(['error' => true, 'message' => 'OTP not matched']);
                }
            } else {
                return response()->json(['error' => true, 'message' => 'Phone number not found, please send OTP again.']);
            }
        } catch (Exception $ex) {

            return response()->json(['error' => true, 'message' => $ex->getMessage()]);
        }


    }

    /*public function sendsms($numbers, $otp)
    {
        // Account details
        $apiKey = '3a2ba461-2911-11ed-9c12-0200cd936042';
        //$apiKey = config('constants.2FACTOR_KEY');

        // Message details
        //$numbers = array(9574702571);
        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => "http://2factor.in/API/V1/" . $apiKey . "/SMS/" . $numbers . "/" . $otp,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_POSTFIELDS => "",
            CURLOPT_HTTPHEADER => array(
                "content-type: application/x-www-form-urlencoded"
            ),
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
            return "cURL Error #:" . $err;
        } else {
            return $response;
        }
    }*/

    public function insertUser()
    {
        $users = OldUser::all();
        $userRole = Role::findByName(User::ROLE_CLIENT);

        foreach ($users as $key => $value) {
            //$dupuser=User::where('phone_number','=',$value->mobile)->first();
            //if(empty($dupuser)){
            $Oldbank = Oldbank::where('user_id', '=', $value->user_id)->first();

            $user = User::create([

                'name' => $value->user_name,
                'email' => $value->email,
                'username' => $value->user_name,
                'password' => bcrypt($value->password),
                'phone_number' => $value->mobile,
                'security_pin' => $value->security_pin,
                'betting' => ($value->betting_status == '1') ? 'yes' : 'no',
                'transfer' => ($value->transfer_point_status == '1') ? 'yes' : 'no',
                'status' => ($value->status == '1') ? 'active' : 'blocked',
                'bank_name' => (!empty($Oldbank)) ? $Oldbank->bank_name : '',
                'branch_address' => (!empty($Oldbank)) ? $Oldbank->branch_address : '',
                'account_holder_name' => (!empty($Oldbank)) ? $Oldbank->ac_holder_name : '',
                'account_number' => (!empty($Oldbank)) ? $Oldbank->ac_number : '',
                'ifsc_code' => (!empty($Oldbank)) ? $Oldbank->ifsc_code : '',
                'available_balance' => $value->wallet_balance,
                'password_string' => $value->password,
                'phonepay_number' => (!empty($Oldbank)) ? $Oldbank->phone_pay_number : '',
                'googlepay_number' => (!empty($Oldbank)) ? $Oldbank->google_pay_number : '',
                'paytm_number' => (!empty($Oldbank)) ? $Oldbank->paytm_number : '',
                'created_at' => $value->insert_date,
                'updated_at' => $value->last_update,
            ]);
            $user->role = $userRole->name;
            $user->save();
            $user->assignRole($userRole);
            //}

        }

    }
}
