<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Api\BaseController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\User;
use App\Models\Transfer;
use App\Models\TransactionHistory;
use App\Models\MainSetting;
use App\Models\Notice;
use App\Models\UserInquiry;
use App\Models\ImageSlide;
use Validator;
use Session;
use Hash;
use Carbon\Carbon;

class UserController extends BaseController
{
    public function editProfile(){
        $user=User::select('id','username','email','phone_number','available_balance','bank_name','account_holder_name','account_number','ifsc_code','phonepay_number','googlepay_number','paytm_number','status','betting')->where('id',\Auth::user()->id)->first();
        $settingValue = MainSetting::select('global_betting')->find(1);
        if ($user && $settingValue) {
            $user->global_on = ($settingValue->global_betting == '1') ? 'yes' : 'no';
        }        
        
        return $this->sendResponse($user,'Profile update succesfully');
    }

    public function updateProfile(Request $request)
    {

        //dd(\Auth::user()->id);
        $validator = Validator::make($request->all(), [
            'username' => 'required',
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors()->toArray();
            $message = "";
            foreach ($errors as $key => $error) {
                $message .= $error[0] . "\n";
            }
            return $this->sendError($message, []);
        }

        $user = User::find(\Auth::user()->id);
        $user->username = $request->username;
        $user->email = $request->email;
        //$user->phone_number = $request->phone_number;

        if ($request->hasfile('profile_pic')) {
            $file = $request->file('profile_pic');
            $actual_name = $request->file('profile_pic')->getClientOriginalName();
            $extension = $request->file('profile_pic')->extension();
            $filename = time() . '_' . $actual_name;
            $file->move(public_path() . '/asset/images/', $filename);

            $user->profile_picture = 'https://game.deeplaxmi.net/asset/images/' . $filename;

        }
        $user->save();

        return $this->sendResponse($user, 'Profile update succesfully');
    }

    public function updateBankDetails(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'bank_name' => 'required',
            //'branch_address'     => 'required',
            'account_holder_name' => 'required',
            'account_number' => 'required',
            'ifsc_code' => 'required',
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors()->toArray();
            $message = "";
            foreach ($errors as $key => $error) {
                $message .= $error[0] . "\n";
            }
            return $this->sendError($message, []);
        }

        $user = User::find(\Auth::user()->id);
        $user->bank_name = $request->bank_name;
        //$user->branch_address=$request->branch_address;
        $user->account_holder_name = $request->account_holder_name;
        $user->account_number = $request->account_number;
        $user->ifsc_code = $request->ifsc_code;

        $user->phonepay_number = $request->phonepay_number;
        $user->googlepay_number = $request->googlepay_number;
        $user->paytm_number = $request->paytm_number;
        $user->save();

        return $this->sendResponse([], 'Bank Details update succesfully');
    }

    public function changePassword(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'old_password' => 'required|min:6',
            'new_password' => 'required|min:6',
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors()->toArray();
            $message = "";
            foreach ($errors as $key => $error) {
                $message .= $error[0] . "\n";
            }
            return $this->sendError($message, []);
        }
        $profile = User::find(\Auth::user()->id);
        if (Hash::check($request->old_password, $profile->password)) {

            $profile->password = bcrypt($request->new_password);
            $profile->password_string = $request->new_password;
            $profile->save();
            return response()->json(['error' => false, 'message' => 'Password updated successfully.']);
        } else {
            return response()->json(['error' => true, 'message' => 'Old password does not matched.']);

        }

    }

    public function forgotPassword(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'phone_number' => 'required',
            'new_password' => 'required|min:6',
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors()->toArray();
            $message = "";
            foreach ($errors as $key => $error) {
                $message .= $error[0] . "\n";
            }
            return $this->sendError($message, []);
        }
        $profile = User::where('phone_number', '=', $request->phone_number)->first();
        if (!empty($profile)) {

            $profile->password = bcrypt($request->new_password);
            $profile->password_string = $request->new_password;

            $profile->save();
            return response()->json(['error' => false, 'message' => 'Password set successfully.']);
        } else {
            return response()->json(['error' => true, 'message' => 'Phone number does not matched with any user.']);
        }
    }

    public function transfer(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'phone_number' => 'required',
            'amount' => 'required',
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors()->toArray();
            $message = "";
            foreach ($errors as $key => $error) {
                $message .= $error[0] . "\n";
            }
            return $this->sendError($message, []);
        }
        $receiver = User::where('phone_number', '=', $request->phone_number)->first();
        if (!empty($receiver)) {

            $receiver->available_balance = $receiver->available_balance + $request->amount;
            $receiver->save();

            $sender = User::find(\Auth::user()->id);
            $sender->available_balance = $sender->available_balance - $request->amount;
            $sender->save();

            TransactionHistory::create([

                'admin_id' => 0,
                'user_id' => \Auth::user()->id,
                'amount' => $request->amount,
                'transaction_note' => 'Transfer to ' . $receiver->username,
                'transfer_note' => 'Transfer to ' . $receiver->username,
                'type' => 'debit',
                'date' => date('Y-m-d H:i:s'),
                'tx_req_number' => random_int(100000, 999999),

            ]);


            TransactionHistory::create([

                'admin_id' => 0,
                'user_id' => $receiver->id,
                'amount' => $request->amount,
                'transaction_note' => 'Receive transfer from ' . $receiver->username,
                'transfer_note' => 'Receive transfer from ' . $receiver->username,
                'type' => 'credit',
                'date' => date('Y-m-d H:i:s'),
                'tx_req_number' => random_int(100000, 999999),

            ]);

            Transfer::create([

                'sender_id' => $sender->id,
                'receiver_id' => $receiver->id,
                'amount' => $request->amount,
                'date' => date('Y-m-d'),
            ]);

            return response()->json(['error' => false, 'message' => 'Transfer done successfully!']);
        } else {
            return response()->json(['error' => true, 'message' => 'No user found for given phone number!']);

        }
    }

    public function settings(Request $request)
    {

        $setting['result'] = MainSetting::first();
        return $this->sendResponse($setting, 'app setting');

    }

    public function notice(Request $request)
    {

        $notice['result'] = Notice::orderBy('id', 'desc')->get();
        return $this->sendResponse($notice, 'User notice.');

    }

    public function slideBanner(Request $request)
    {

        $notice['result'] = ImageSlide::where('status', '=', 'active')->orderBy('image_order', 'asc')->get();
        return $this->sendResponse($notice, 'slider image');

    }

    public function userInquiry(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'phone_number' => 'required',
            'email' => 'required',
            'name' => 'required',
            'msg' => 'required',
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors()->toArray();
            $message = "";
            foreach ($errors as $key => $error) {
                $message .= $error[0] . "\n";
            }
            return $this->sendError($message, []);
        }

        $inquiry = UserInquiry::create([

            'name' => $request->name,
            'email' => $request->email,
            'phone_number' => $request->phone_number,
            'msg' => $request->msg,
        ]);
        $inquiry->save();
        return response()->json(['error' => false, 'message' => 'Inquiry submitted successfully.']);

    }

    public function deleteUser(Request $request){
        $user = User::find($request->id);
        if($user){
            $user->delete();
        }

        $user->status = 'blocked';
        $user->save();        

        return response()->json(['error' => false, 'message' => 'User Deleted successfully']);
    }

    public function mainsetting(Request $request){
        $mainSetting = MainSetting::find(1);
        return $this->sendResponse($mainSetting,'Main settings succesfully');
    }

    public function verifyphone(Request $request){
        $count = User::where('phone_number',$request->phone_number)->count();
        if($count){
            return $this->sendResponse([],'phone number found');
        } else {
            return $this->sendError('Phone number does not exists');
        }
    }

}
