<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Api\BaseController;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\UserBid;
use App\Models\Games;
use App\Models\GameType;
use App\Models\Withdraw;
use App\Models\MainSetting;
use App\Models\TransactionHistory;
use App\Events\UpdateBalance;
use Validator;
use Session;
use Auth;
use DB;
use Carbon\Carbon;

class WalletController extends BaseController
{
    public function addFund(Request $request){
       
       if($request->payment_method == '1' || $request->payment_method == '3'){
       
	       $fund=TransactionHistory::create([
		
		    'admin_id' => 0,
		    'user_id' => \Auth::user()->id,
		    'amount' => $request->amount,
		    'transaction_note' => 'Points added via UPI Tx ID '.$request->transaction_id,
		    'transfer_note' => '',
		    'type' => 'credit',
		    'auto_deposit' => '1',
		    'transaction_id' => $request->transaction_id,
		    'payment_method' => $request->payment_method,
		    'date' => date('Y-m-d H:i:s'),
		    'tx_req_number' => random_int(100000, 999999),
		    'status' => '2',
		
		]);
		
       }else{
       
	       $user=User::find(\Auth::user()->id);
	       $user->available_balance=$user->available_balance + $request->amount;
	       $user->save();

           broadcast(new UpdateBalance($user->available_balance,$user->id));
	       
	       $fund=TransactionHistory::create([
		
		    'admin_id' => 0,
		    'user_id' => \Auth::user()->id,
		    'amount' => $request->amount,
		    'transaction_note' => 'Points added via UPI Tx ID '.$request->transaction_id,
		    'transfer_note' => '',
		    'type' => 'credit',
		    'auto_deposit' => '1',
		    'transaction_id' => $request->transaction_id,
		    'payment_method' => $request->payment_method,
		    'date' => date('Y-m-d H:i:s'),
		    'tx_req_number' => random_int(100000, 999999),
		    'status' => '1',
		
		]);
		
       }
       return $this->sendResponse([],'Fund added succesfully');
    } 
    public function WalletsNumber(Request $request){
    
       
        $user['result']=User::find(\Auth::user()->id);
        if($request->wallet_type == '1'){
            $user['result']->paytm_number=$request->phone_number;
        }elseif($request->wallet_type == '2'){
            $user['result']->googlepay_number=$request->phone_number;
        }elseif($request->wallet_type == '3'){
            $user['result']->phonepay_number=$request->phone_number;
        }
        $user['result']->save();
        return $this->sendResponse($user,'Number updated succesfully');
    }
    public function withdrawRequest(Request $request){

        $setting=MainSetting::first();
        if($request->amount < $setting->min_withdrawal){
           
            $data ='Withdraw points should be greater than '.$setting->min_withdrawal;
            return $this->sendError($data);
        }
        if($request->amount > $setting->max_withdrawal){
            
            $data ='Withdraw points should be less than '.$setting->max_withdrawal;
            return $this->sendError($data);
        }

        if($request->payment_method == ''){
            
            $data ='please select payment method';
            return $this->sendError($data);
        }

        if($request->amount > \Auth::user()->available_balance){
            
            $data ='Please select amount less than wallet balance';
            return $this->sendError($data);
        }

        if($request->payment_method == 'PaytmNumber'){
            $payment_method = '1';
            $number=\Auth::user()->paytm_number;
        }else if($request->payment_method == 'GooglePay'){
            $payment_method = '2';
            $number=\Auth::user()->googlepay_number;
        }else{
            $payment_method = '3';
            $number=\Auth::user()->phonepay_number;

        }
    
       $check = Withdraw::where('user_id','=', \Auth::user()->id)->where('status','=','3')->count();
       if($check > 0){
        return $this->sendError('Prevoius withdraw request already in process, You can`t make new request.',[]);  

       }

       $withdraw=Withdraw::create([
        
            'user_id' => \Auth::user()->id,
            'amount' => $request->amount,
            'request_date' => date('Y-m-d H:i:s'),
            'transfer_number' => $number,
            'payment_method' => $payment_method,
            'status' => '3',
            'request_number' => random_int(100000, 999999),
        
        ]);
        $withdraw->save();
        
        $user=User::find(\Auth::user()->id);
        $user->available_balance=$user->available_balance - $request->amount;
        $user->save();
        broadcast(new UpdateBalance($user->available_balance,$user->id));
        //$this->fcm('refresh','',$user->device_token);
        return $this->sendResponse(["wallet" => $user->available_balance],'Withdraw request send succesfully');
    }
    public function withdrawHistory(Request $request){
    
        $check['result'] = Withdraw::where('user_id','=', \Auth::user()->id)->orderBy('id','desc')->get()->toArray();
        return $this->sendResponse($check,'Withdraw history');
    }
    public function transferUserList(Request $request){
    
        $check['result'] = User::select(DB::raw('username,phone_number'))->get()->toArray();
        return $this->sendResponse($check,'user list');
    }

    public function getTransactionHistory(Request $request){
     	$history = TransactionHistory::select('transaction_history.*')
     		->where('user_id',auth()->user()->id)
            ->orderByDesc('id')
     		->get();

     	return $this->sendResponse($history,'Transaction History history');	
    }
    public function getpaymentmethods(Request $request){

    	$user = User::find(auth()->user()->id);

    	$methods = array(
    		array(
    			'method' => 'GooglePay',
    			'payid' => $user->googlepay_number,
    		),
    		array(
    			'method' => 'PhonepayNumber',
    			'payid' => $user->phonepay_number,
    		),
    		array(
    			'method' => 'PaytmNumber',
    			'payid' => $user->paytm_number,
    		),
    	);
    	return $this->sendResponse($methods,'list of payment methods');
    }

    public function withdraw(Request $request){

       $setting=MainSetting::first();
       if($request->amount < $setting->min_withdrawal){
            $data['error'] =true;
            $data['message'] ='Withdraw points should be greater than '.$setting->min_withdrawal;
            return json_encode($data);
       }
       if($request->amount > $setting->max_withdrawal){
            $data['error'] =true;
            $data['message'] ='Withdraw points should be less than '.$setting->max_withdrawal;
            return json_encode($data);
       }

       if($request->payment_method == 'PaytmNumber'){
        $number=\Auth::user()->paytm_number;
       }else if($request->payment_method == 'GooglePay'){
       $number=\Auth::user()->googlepay_number;
       }else{
       $number=\Auth::user()->phonepay_number;

       }

       $response=$this->apiPostRequest('withdrawRequest',array("payment_method" => $request->payment_method,"amount" => $request->amount,'transfer_number' =>$number));
       if($response->error == false){

           return response()->json(['error' => false, 'message' => 'success']);

       }else{
              return response()->json(['error' => true, 'message' => $response->message]);
       }

    }
    
}
